import { mount, createLocalVue } from '@vue/test-utils'
import ClInput from '@/components/ClInput.vue'
import ElementUI from 'element-ui'
import SwordUi from '@/index'

const localVue = createLocalVue()
//使用了Element的组件
localVue.use(ElementUI)
//使用了Sword的组件
localVue.use(SwordUi)

describe('ClInput.vue', () => {
  const wrapper = mount(ClInput, {
    localVue,
    propsData: {
      value: null,
      permission: 'b',
    },
  })
  //切换验证的提示信息为中文
  wrapper.vm.$validator.localize('zh_CN')
  //找到input元素
  const elInput = wrapper.find("input[type='text']")

  it('Required Validation Before', (done) => {
    //需要触发失去焦点事件，才会触发表单必填
    elInput.trigger('blur')
    //异步中校验必填提示是否显示出来了
    wrapper.vm.$nextTick(() => {
      expect(wrapper.vm.errors.items.length).toBe(1)
      expect(wrapper.vm.errors.items[0].msg).toEqual('必填')
      //异步校验中，必须执行done回调，否则单元测试进程会被悬挂
      done()
    })
  })

  it('Required Validation After', (done) => {
    //设置输入框的值
    wrapper.setData({ value: '张三' })
    wrapper.vm.$validator.validateAll().then((result) => {
      expect(wrapper.vm.errors.items.length).toBe(0)
      done()
    })
  })
})
