import FormMath from '@/math.js'

describe('FormMath', () => {
  it('parseFuncexpField', () => {
    const exp = `{单价(goods.price)} * {数量(goods.quantity)}`
    expect(FormMath.parseFuncexpField(exp)[1]).toEqual('goods.quantity')
  })

  it('calcSubExpValue', () => {
    const exp = `FormMath.tofixed({小计(item.total)} / {数量(item.quantity)}, 2)`,
      scope = {
        data: {
          sub: [
            { quantity: 3, total: 205210 },
            { quantity: 2, total: 6838 },
          ],
        },
      },
      prefix = 'data.sub',
      index = 0
    const result = FormMath.calcSubExpValue(exp, scope, prefix, index)
    expect(result).toEqual((205210 / 3).toFixed(2))
  })

  it('calcExpValue', () => {
    const exp = `FormMath.sum([{小计(data.sub.total)}])`,
      scope = {
        data: {
          math1: '2030',
          math2: 1230,
          sub: [
            { quantity: 3, total: 205210 },
            { quantity: 2, total: 6838 },
          ],
        },
      }
    expect(FormMath.calcExpValue(exp, scope)).toEqual(
      (205210 + 6838).toFixed(0)
    )

    const exp2 = `{数字1(data.math1)} - {数字2(data.math2)}`
    expect(FormMath.calcExpValue(exp2, scope)).toEqual((2030 - 1230).toFixed(0))
  })
})
