import utils from '@/utils.js'

describe('utils', () => {
  it('getName', () => {
    expect(utils.getName().length > 8).toBeTruthy()
  })
  it('stringSplit', () => {
    expect(utils.stringSplit(', dj , dwq,,', ',')).toHaveLength(2)
  })
  it('mergeValidate', () => {
    const rules = 'max:10 | min:5',
      append1 = 'between:1,10',
      append2 = {
        method: (value) => {
          return value == 'test'
        },
      }
    expect(utils.mergeValidate(rules, append1)).toContain(append1)
    let r = utils.mergeValidate(rules, append2)
    expect(Object.keys(r)).toHaveLength(3)
  })
  it('addRequiredOrNot', () => {
    expect(utils.addRequiredOrNot('b', { max: 50 })).toHaveProperty(
      'required',
      true
    )
    expect(utils.addRequiredOrNot('b', 'max:50|min:10')).toContain('required')
  })
  it('formatDate', () => {
    const dateTimeStr = '2019-11-13 19:41:00'
    expect(utils.formatDate(new Date(dateTimeStr))).toEqual(dateTimeStr)
  })
  it('formatDateYear', () => {
    const dateStr = '2019-11-13'
    expect(utils.formatDateYear(new Date(dateStr))).toEqual(dateStr)
  })
  it('dateIsBefore', () => {
    expect(
      utils.dateIsBefore('2019-11-13 19:41:00', '2019-11-13 19:41:00', true)
    ).toBeTruthy()
    expect(
      utils.dateIsBefore('2019-11-13 19:41:00', '2019-11-13 19:41:00')
    ).toBeFalsy()
    expect(utils.dateIsBefore('2019-11-14', '2019-11-13')).toBeFalsy()
  })
  it('dateCalc', () => {
    expect(utils.dateCalc('2018-06-13', '2019-06-13', 'year')).toEqual(1)
    expect(utils.dateCalc('2019-06-13', '2019-11-14', 'month')).toEqual(5)
    expect(
      utils.dateCalc('2019-11-13 19:40:00', '2019-11-14 19:40:00')
    ).toEqual(1)
    expect(
      utils.dateCalc('2019-11-13 19:40:00', '2019-11-14 19:40:00', 'hour')
    ).toEqual(24)
    expect(
      utils.dateCalc('2019-11-13 19:40:00', '2019-11-14 19:40:00', 'minute')
    ).toEqual(1440)
    expect(
      utils.dateCalc('2019-11-14 19:40:00', '2019-11-14 19:42:00', 'second')
    ).toEqual(120)
  })
  it('tile2nest', () => {
    const tile = [
      { id: '1', parentId: '-1', name: '1' },
      { id: '2', parentId: '1', name: '1-2' },
      { id: '3', parentId: '2', name: '1-2-3' },
      { id: '4', parentId: '2', name: '1-2-4' },
      { id: '5', parentId: '1', name: '1-5' },
      { id: '6', parentId: '5', name: '1-5-6' },
    ]
    let nest = utils.tile2nest(tile)
    expect(nest).toHaveLength(1)
    expect(nest[0].children).toHaveLength(2)
    expect(nest[0].children[1].children[0].name).toEqual('1-5-6')
    const tile2 = [
      { code: 'a', pCode: 'x', name: 'a' },
      { code: 'b', pCode: 'a', name: 'a-b' },
      { code: 'c', pCode: 'b', name: 'a-b-c' },
    ]
    expect(
      utils.tile2nest(tile2, 'code', 'pCode', 'sub')[0].sub[0].sub[0].name
    ).toEqual('a-b-c')
  })
  it('groupByKey', () => {
    const ary = [
      {
        property: 'name',
        operation: 'like',
        value: '张三',
        group: 'advance',
        relation: 'AND',
      },
      {
        property: 'status',
        operation: 'equal',
        value: '张三',
        group: 'filter',
        relation: 'AND',
      },
      {
        property: 'name',
        operation: 'equal',
        value: '张三',
        group: 'filter',
        relation: 'AND',
      },
      {
        property: 'account',
        operation: 'like',
        value: 'zhangsan',
        group: 'advance',
        relation: 'AND',
      },
      {
        property: 'name',
        operation: 'like',
        value: 'san',
        group: 'quick',
        relation: 'OR',
      },
      {
        property: 'account',
        operation: 'like',
        value: 'san',
        group: 'quick',
        relation: 'OR',
      },
      {
        property: 'mobile',
        operation: 'like',
        value: 'san',
        group: 'quick',
        relation: 'OR',
      },
    ]
    const group = ary.groupByKey('group')
    expect(Object.keys(group).length).toBe(3)
    expect(group['quick'].length).toBe(3)
  })
})
